# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:18
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :gameutils.py

def bold(string: str) -> str:
    """Print a string in 'bold' font"""
    return f"\033[1m{string}\033[0m"



# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:23
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :abstractgameunit.py

import random
from abc import ABCMeta, abstractmethod
from gameutils import bold
from gameuniterror import HealthMeterException

class AbstractGameUnit(metaclass=ABCMeta):
    def __init__(self, name=''):
        self.name = name
        self.unit_type = None
        self.health_meter = None
        self.max_hp = None

    @abstractmethod
    def info(self):
        pass

    def attack(self, opponent):
        hurt_one = random.choice([self, opponent])
        hurt_one.health_meter = hurt_one.health_meter - random.randint(10, 15)
        print('ATTACK!', end='')

    def heal(self, heal_by=2, full_healing=True):
        """Heal the unit replenishing all the hit points"""
        if self.health_meter == self.max_hp:
            return
        if full_healing:
            self.health_meter = self.max_hp
        else:
            self.health_meter += heal_by
        # -------------------------------------------------------------------
        # raise a custom exception. Refer to chapter on exception handling
        # -------------------------------------------------------------------
        if self.health_meter > self.max_hp:
            raise HealthMeterException("health_meter > max_hp!")

        print(bold(f"You are HEALED!"), end=" ")
        self.show_health(bold_str=True, end='\r\n')

    def reset_health_meter(self):
        pass

    def show_health(self, bold_str=False, end=''):
        health_show = f"Health: {self.name}: {self.health_meter} "
        if bold_str:
            health_show = bold(health_show)
        print(health_show, end=end)


# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:28
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :attackoftheorcs.py

import random
from hut import Hut
from knight import Knight
from orcrider import OrcRider
from gameutils import bold
from typing import List
from gameuniterror import HutError


class AttackOfTheOrcs:
    def __init__(self):
        self.huts: List[Hut] = []
        self.player = None

    def get_occupants(self):
        current_occupants = []
        for hut in self.huts:
            current_occupants.append(hut.get_occupant_type())
        return str(current_occupants)

    def show_game_mission(self):
        print(bold('Mission:'))
        print('\t1. Fight with the enemy.')
        print('\t2. Bring all the huts in the village under your control.')
        print('-' * 72)
        print()

    def _process_user_choice(self) -> int:
        """Process the user input for choice of hut to enter"""
        verifying_choice = True
        idx = 0
        print(f'Current occupants:{self.get_occupants()}')
        while verifying_choice:
            try:
                user_choice = input('Choose a hut number to enter (1-5): ')
                try:
                    idx = int(user_choice)
                except ValueError:
                    raise HutError(103)
                try:
                    if self.huts[idx - 1].isacquired:
                        raise HutError(104)
                except IndexError:
                    raise HutError(101)
                else:
                    if idx not in range(1, 6):
                        raise HutError(102)
            except HutError:
                continue
            else:
                verifying_choice = False

        return int(idx)

    def _occupy_huts(self):
        """Randomly occupy the huts with one of: friend enemy or 'None'"""
        for i in range(5):
            choice_lst = ['enemy', 'friend', None]
            computer_choice = random.choice(choice_lst)
            if computer_choice == 'enemy':
                name = 'enemy-' + str(i + 1)
                self.huts.append(Hut(i + 1, OrcRider(name)))
            elif computer_choice == 'friend':
                name = 'knight-' + str(i + 1)
                self.huts.append(Hut(i + 1, Knight(name)))
            else:
                self.huts.append(Hut(i + 1, computer_choice))

    def play(self):
        self.player = Knight()
        self._occupy_huts()
        acquired_hut_counter = 0

        self.show_game_mission()
        self.player.show_health(bold_str=True, end='\r\n')

        while acquired_hut_counter < 5:
            idx = self._process_user_choice()
            self.player.acquire_hut(self.huts[idx - 1])
            if self.player.health_meter <= 0:
                print(bold("YOU LOSE :( Better luck next time"))
                break

            if self.huts[idx - 1].isacquired:
                acquired_hut_counter += 1

        if acquired_hut_counter == 5:
            print(bold("Congratulations! YOU WIN!!!"))


if __name__ == '__main__':
    game = AttackOfTheOrcs()
    game.play()



# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:27
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :orcrider.py

from abstractgameunit import AbstractGameUnit


class OrcRider(AbstractGameUnit):
    def __init__(self, name):
        AbstractGameUnit.__init__(self)
        self.unit_type = 'enemy'
        self.name = name
        self.health_meter = 30
        self.max_hp = 30

    def info(self):
        pass



# -*- coding: utf-8 -*-
# @Time    :2021/3/8 10:48
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :gameuniterror.py.py

class GameUnitError(Exception):
    """Custom exception class for the 'AbstractGameUnit' and its subclasses"""

    def __init__(self, message=''):
        super().__init__(message)
        self.padding = '~' * 50 + '\r\n'
        self.error_message = "Unspecified Error!"


class HealthMeterException(GameUnitError):
    """Custom exception to report Health Meter related problems"""

    def __init__(self, message=''):
        super(HealthMeterException, self).__init__(message)
        self.error_message = (self.padding +
                              "ERROR: Health Meter Problem" +
                              '\r\n' + self.padding)


class HutError(Exception):
    def __init__(self, code):
        self.error_message = ''
        self.error_dict = {
            000: "E000: Unspecified Error code",
            101: "E101: Out of range: Number > 5",
            102: "E102: Out of range, Negative number",
            103: "E103: Not a number!",
            104: "E104: You have already acquired this hut. Try again."
                 "<INFO: You can NOT get healed in already acquired hut.>",
        }
        try:
            self.error_message = self.error_dict[code]
        except KeyError:
            self.error_message = self.error_dict[000]
        print("Error message: ",self.error_message)



# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:26
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :knight.py

from abstractgameunit import AbstractGameUnit
from gameutils import bold

class Knight(AbstractGameUnit):
    def __init__(self, name='Sir Foo'):
        AbstractGameUnit.__init__(self)
        self.name = name
        self.health_meter = 40
        self.max_hp = 40
        self.unit_type = 'friend'

    def info(self):
        pass

    def acquire_hut(self, hut):
        """Fight the combat (command line) to acquire the hut"""
        print(bold(f"Entering hut {hut.number},"), end=' ')
        is_enemy = (isinstance(hut.occupant, AbstractGameUnit)
                    and hut.occupant.unit_type == 'enemy')
        continue_attack = 'y'
        if is_enemy:
            print(bold("Enemy sighted!"))
            self.show_health(bold_str=True)
            hut.occupant.show_health(bold_str=True)
            while continue_attack:
                continue_attack = input("......continue attack?(y/n):")
                if continue_attack == 'n':
                    self.run_away()
                    break

                self.attack(hut.occupant)

                if hut.occupant.health_meter <= 0:
                    print("")
                    hut.acquire(self)
                    break
                if self.health_meter <= 0:
                    print("")
                    break

                self.show_health()
                hut.occupant.show_health()
        else:
            if hut.get_occupant_type() == 'unoccupied':
                print(bold('Hut is unoccupied'))
            else:
                print(bold("Friend sighted!"))
            hut.acquire(self)
            self.heal()

    def run_away(self):
        print(bold("RUNNING AWAY"))



# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:25
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :hut.py

from gameutils import bold


class Hut:
    """Class to create hut object(s) in the game Attack of the Ocrs"""

    def __init__(self, number, occupant):
        self.occupant = occupant
        self.number = number
        self.isacquired = False

    def acquire(self, new_occupant):
        """Update the occupant of the hut"""
        self.occupant = new_occupant
        self.isacquired = True
        print(bold(f"GOOD JOB! Hut {self.number} acquired"))

    def get_occupant_type(self):
        """Return a string giving info on the hut occupant"""
        if self.isacquired:
            occupant_type = "ACQUIRED"
        elif self.occupant is None:
            occupant_type = 'unoccupied'
        else:
            occupant_type = self.occupant.unit_type

        return occupant_type



from distutils.core import setup

with open('README.rst') as file:
    readme = file.read()

setup(
    name='orcwargame',
    version='0.1.0',
    packages=['src.orcwargame'],
    url='https://pypi.org/project/orcwargame/0.1.0/',
    license='LICENSE.txt',
    description='a war game',
    long_description=readme,
    author='MaLiang',
    author_email='maliang@email.cn'
)



# -*- coding: utf-8 -*-
# @Time    :2021/3/8 14:57
# @Author  :Ma Liang
# @Email   :mal818@126.com
# @File    :__init__.py.py

import sys
import os
current_path = os.path.dirname(os.path.abspath(__file__))
sys.path.append(current_path)
